// TubeControl.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "TubeControl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// TubeControl

TubeControl::TubeControl(const std::vector<Tube>* tube_list) : m_control_size(0, 0)
{  
  COLOR_NOT_EXIST = RGB(200,200,200);
  COLOR_NOT_ANNOTATED = RGB(255,0,0);
//  COLOR_NOT_DONE = RGB(255,255,0);
  COLOR_DONE = RGB(0,255,0);

  m_tube_list = tube_list;
  //m_session = 1;
  //m_control_margin = m_tube_spacing = 15;
  //m_tube_margin = m_window_spacing = 3;
  //m_tube_top = 10;
  m_window_size = CSize(20, 15);
  //m_row_length = 1;
  //m_column_length = 2;
  //m_label_height = 12;

  //m_tube_size = CSize(m_window_size.cx * m_row_length + (m_row_length - 1) * m_window_spacing + 2 * m_tube_margin,
  //  m_tube_top * 1.5 + m_window_size.cy * m_column_length + (m_column_length - 1) * m_window_spacing + 2 * m_tube_margin);

  m_tool_tip.Create(this);
}

TubeControl::~TubeControl()
{
}

void TubeControl::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CStatic::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(TubeControl, CStatic)
	//{{AFX_MSG_MAP(TubeControl)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(TubeControl, CStatic)
	//{{AFX_DISPATCH_MAP(TubeControl)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_ITubeControl to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {D9B9BB98-A043-427E-A6F1-F755D2B1211B}
static const IID IID_ITubeControl =
{ 0xd9b9bb98, 0xa043, 0x427e, { 0xa6, 0xf1, 0xf7, 0x55, 0xd2, 0xb1, 0x21, 0x1b } };

BEGIN_INTERFACE_MAP(TubeControl, CStatic)
	INTERFACE_PART(TubeControl, IID_ITubeControl, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// TubeControl message handlers

void TubeControl::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
  int tube_index;
  CRect rect;
  GetClientRect(rect);
  if (m_control_size != rect.Size())
  {
    m_control_size = rect.Size();
  }

  DrawLegend(&dc);

  //Determine index of current tube in tube vector
  for(int i = 0; i < m_tube_list->size(); i++)
  {
	  if (m_tube_list->at(i).GetTag() == m_tube)
		  tube_index = i;
  }

  int col_length = 22;
  int row_length = 17;
  int columns = 27;
  int rows = 15;
  for(i = 1; i<columns; i++)
  {
	  for(int j = 1; j<rows; j++)
	  {
      //UPDATED PROGRESS WINDOW CHECKS
      //ASDF

      //First check if image exists
      if(!(m_tube_list->at(tube_index).GetImageExists(j, i))){
        DrawWindow(WindowRect(CPoint((i*col_length), (j*row_length)), m_window_size, 1), COLOR_NOT_EXIST, &dc);
      
      //Second Check if image is annotated
      } else if(m_tube_list->at(tube_index).GetImageAnnotated(j, i)){
		    DrawWindow(WindowRect(CPoint((i*col_length), (j*row_length)), m_window_size, 1), COLOR_DONE, &dc);

      //Else the image is not annotated
      } else {
        DrawWindow(WindowRect(CPoint((i*col_length), (j*row_length)), m_window_size, 1), COLOR_NOT_ANNOTATED, &dc);
      }
    }
  }
	
	// TODO: Add your message handler code here
	
	// Do not call CStatic::OnPaint() for painting messages
}

void TubeControl::SetSession(const int &session)
{
  m_session = session;
}

void TubeControl::SetTube(const CString &tube)
{
  m_tube = tube;
}

TubeControl::WindowRect::WindowRect() : CRect()
{
  window = 0;
}

TubeControl::WindowRect::WindowRect(const CPoint &point, const CSize &size, const int &window) : CRect(point, size)
{
  this->window = window;
}

TubeControl::TubeRect::TubeRect() : CRect()
{
  tube = 0;
  windows = std::vector<WindowRect>();
}

TubeControl::TubeRect::TubeRect(const CPoint &point, const CSize &size, const int &tube) : CRect(point, size)
{
  this->tube = tube;
  windows = std::vector<WindowRect>();
}

void TubeControl::DrawWindow(const WindowRect &window, const COLORREF &color, CPaintDC *dc)
{
  // !!! change this to FrameRect()
	//draw a rectangle
	dc->MoveTo(window.left, window.top);
	dc->LineTo(window.left, window.bottom);
	dc->LineTo(window.right, window.bottom);
	dc->LineTo(window.right, window.top);
	dc->LineTo(window.left, window.top);

  //color the rectangle
  dc->FillSolidRect(CRect(window.TopLeft() + CSize(1,1), window.Size() - CSize(1,1)), color);
}

void TubeControl::OnLButtonDown(UINT nFlags, CPoint point) 
{
/*  int tube, window;
  CRect rect;

  GetAt(point, &tube, &window, &rect);

  if ((tube != -1) && (window != -1) && (m_data->GetExists(tube, window, m_session)))
  {
    SelectImage(tube, m_session, window);
  }
	
	CStatic::OnLButtonDown(nFlags, point);*/
}

void TubeControl::GetAt(CPoint point, int *tube, int *window, CRect *rect)
{
  int t, w;

  for (t = 0; t < m_tubes.size(); t++)
  {
    if (!m_tubes[t].PtInRect(point))
    {
      continue;
    }

    for (w = 0; w < m_tubes[t].windows.size(); w++)
    {
      if (m_tubes[t].windows[w].PtInRect(point))
      {
        *tube = m_tubes[t].tube;
        *window = m_tubes[t].windows[w].window;
        *rect = m_tubes[t].windows[w];
        return;
      }
    }
  }

  *tube = *window = -1;
}

void TubeControl::SetImageSelectCallback(TubeControl::ImageSelectCallback *callback)
{
  m_select_image = callback;
}

// this set up may seem overly complicated, but it does avoid required TubeControl to have knowledge of ProgressDlg
void TubeControl::SelectImage(const int &tube, const int &session, const int &window)
{
  m_select_image->SelectImage(tube, session, window);
}

void TubeControl::DrawLegend(CPaintDC *dc)
{
  dc->MoveTo(500,370);
  dc->LineTo(520,370);
  dc->LineTo(520, 385);
  dc->LineTo(500, 385);
  dc->LineTo(500,370);
  CRect done;
  done.SetRect(500,370,520,385);
  dc->FillSolidRect(done, COLOR_DONE);

  dc->MoveTo(500,390);
  dc->LineTo(520,390);
  dc->LineTo(520, 405);
  dc->LineTo(500, 405);
  dc->LineTo(500,390);
  CRect empty;
  empty.SetRect(500,390,520,405);
  dc->FillSolidRect(empty, COLOR_NOT_ANNOTATED);
  
  dc->MoveTo(500,410);
  dc->LineTo(520,410);
  dc->LineTo(520, 425);
  dc->LineTo(500, 425);
  dc->LineTo(500,410);
  CRect noImage;
  noImage.SetRect(500,410,520,425);
  dc->FillSolidRect(noImage, COLOR_NOT_EXIST);
}

void TubeControl::OnMouseMove(UINT nFlags, CPoint point) 
{
/*  CRect rect;
  int tube, window;

  GetAt(point, &tube, &window, &rect);
  if ((tube > 0) && (window > 0) && m_data->GetExists(tube, window, m_session))
  {
    CString text;
    text.Format("Window %d", window);
    m_tool_tip.AddTool(this, text);
    m_tool_tip.Activate(true);
  }
  else
  {
    m_tool_tip.Activate(false);
  }

	CStatic::OnMouseMove(nFlags, point);*/
}

BOOL TubeControl::PreTranslateMessage(MSG* pMsg) 
{
  m_tool_tip.RelayEvent(pMsg);
	
	return CStatic::PreTranslateMessage(pMsg);
}
